package resolvers

import (
	"github.com/graphql-go/graphql"
	"gitlab.okta-solutions.com/mashroom/backend/account/api"
	"gitlab.okta-solutions.com/mashroom/backend/auth/api"
	"gitlab.okta-solutions.com/mashroom/backend/common/graphql/util"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/connections"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/resolvers/errs"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/schema"
)

func init() {
	query := graphql.NewObject(graphql.ObjectConfig{
		Name:        "AccountQuery",
		Description: "Account query root",
		Fields: graphql.Fields{
			"info": &graphql.Field{
				Description: "Returns current user profile info",
				Resolve:     AccountQueryInfoResolver,
				Type:        util.BindOutput(account.Account{}, "Authenticated user profile information", nil),
			},
			"whois": &graphql.Field{
				Description: "Returns other user limited profile info",
				Args:        util.BindArguments(account.GetRequest{}),
				Resolve:     AccountQueryWhoisResolver,
				Type:        util.BindOutput(account.Account{}, "Authenticated user profile information", nil),
			},
			"get": &graphql.Field{
				Description: "Get account by id",
				Args:        util.BindArguments(account.GetRequest{}),
				Resolve:     AccountQueryGetResolver,
				Type:        util.BindOutput(account.GetResponse{}, "Account details", nil),
			},
			"list": &graphql.Field{
				Description: "Get account list",
				Resolve:     AccountQueryListResolver,
				Type:        util.BindOutput(account.ServiceListResponse{}, "Account list", nil),
			},
		},
	})

	schema.RootQueries["account"] = &graphql.Field{
		Type: query,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return query, nil
		},
	}

	mutation := graphql.NewObject(graphql.ObjectConfig{
		Name:        "AccountMutation",
		Description: "Account mutation root",
		Fields: graphql.Fields{
			"favorite": &graphql.Field{
				Description: "Favorites an object",
				Resolve:     AccountMutationFavoriteResolver,
				Args:        util.BindArguments(account.FavoriteRequest{}),
				Type:        util.BindOutput(account.FavoriteResponse{}, "Authenticated user profile information", nil),
			},
			"unfavorite": &graphql.Field{
				Description: "Unfavorites an object",
				Args:        util.BindArguments(account.UnfavoriteRequest{}),
				Resolve:     AccountMutationUnfavoriteResolver,
				Type:        util.BindOutput(account.UnfavoriteResponse{}, "Authenticated user profile information", nil),
			},
			"delete": &graphql.Field{
				Description: "Delete account",
				Args:        util.BindArguments(account.DeleteAccountRequest{}),
				Resolve:     AccountMutationDeleteResolver,
				Type:        util.BindOutput(account.DeleteAccountResponse{}, "Delete response", nil),
			},
			"restore": &graphql.Field{
				Description: "Restore account",
				Args:        util.BindArguments(account.RestoreAccountRequest{}),
				Resolve:     AccountMutationRestoreResolver,
				Type:        util.BindOutput(account.RestoreAccountResponse{}, "Restore response", nil),
			},
			"changebankdetails": &graphql.Field{
				Description: "Change bank details of the account",
				Args:        util.BindArguments(account.ChangeBankDetailsRequest{}),
				Resolve:     AccountMutationChangeBankDetailsResolver,
				Type:        util.BindOutput(account.ChangeBankDetailsResponse{}, "Change bank details response", nil),
			},
		},
	})

	schema.RootMutations["account"] = &graphql.Field{
		Type: mutation,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return mutation, nil
		},
	}
}

func AccountMutationChangeBankDetailsResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}

	request := &account.ChangeBankDetailsRequest{}

	err := util.UnmarshalArguments(params.Args, request)
	if err != nil {
		return nil, err
	}

	if remote, ok := params.Context.Value("remote").(string); ok {
		request.Remote = remote
	}
	request.User = user.GetAccount().GetIdValue()

	return connections.AccountService.ChangeBankDetails(params.Context, request)
}

func AccountQueryInfoResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	resp, err := connections.AccountService.Get(params.Context, &account.GetRequest{
		Id:    user.GetAccount().GetIdValue(),
		Admin: false,
	})
	return resp.GetAccount(), err
}

func AccountQueryWhoisResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &account.GetRequest{
		Admin: false,
	}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	resp, err := connections.AccountService.Get(params.Context, request)
	if err != nil {
		return nil, err
	}
	return &account.Account{
		Id:        resp.GetAccount().GetId(),
		CreatedAt: resp.GetAccount().GetCreatedAt(),
		Details:   resp.GetAccount().GetDetails(),
	}, nil
}

func AccountMutationFavoriteResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &account.FavoriteRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()

	return connections.AccountService.Favorite(params.Context, request)
}

func AccountMutationUnfavoriteResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &account.UnfavoriteRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()

	return connections.AccountService.Unfavorite(params.Context, request)
}

func AccountQueryGetResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := CheckPermissions(params.Context, AdminRole); err != nil {
		return nil, err
	}
	request := &account.GetRequest{
		Admin: false,
	}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.AccountService.Get(params.Context, request)
}

func AccountQueryListResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := CheckPermissions(params.Context, AdminRole); err != nil {
		return nil, err
	}
	request := &account.ServiceListRequest{
		Admin: false,
	}
	return connections.AccountService.ServiceList(params.Context, request)
}

func AccountMutationDeleteResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &account.DeleteAccountRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()
	return connections.AccountService.DeleteAccount(params.Context, request)
}

func AccountMutationRestoreResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &account.RestoreAccountRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()
	return connections.AccountService.RestoreAccount(params.Context, request)
}
