package admin

import (
	"github.com/graphql-go/graphql"
	auth "gitlab.okta-solutions.com/mashroom/backend/auth/api"
	"gitlab.okta-solutions.com/mashroom/backend/cardpayment/api"
	q "gitlab.okta-solutions.com/mashroom/backend/common/api/generic/query"
	"gitlab.okta-solutions.com/mashroom/backend/common/graphql/util"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/connections"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/resolvers"
)

type confirmCardpaymentRequest struct {
	ChargeId string `json:"charge_id"`
	Id       string `json:"id"`
	Confirm  bool   `json:"confirm"`
}

type allCardpaymentsResult struct {
	Data         []*cardpayment.ReceiptItem `json:"data,omitempty"`
	Total        int32                      `json:"total,omitempty"`
	NonConfirmed int32                      `json:"non_confirmed, omitempty"`
}

func initCardPayment(query, mutation *graphql.Fields) {
	(*query)["Cardpayment"] = &graphql.Field{
		Description: "Retrieves specified cardpayment item",
		Args:        util.BindArguments(cardpayment.ReceiptItemRequest{}),
		Resolve:     adminQueryGetCardpaymentResolver,
		Type:        util.BindOutput(cardpayment.ReceiptItem{}, "Receipt item information", nil),
	}
	(*query)["allCardpayments"] = &graphql.Field{
		Args:        util.BindArguments(allParams{}),
		Description: "Get all receipts",
		Resolve:     adminQueryAllCardpaymentsResolver,
		Type:        util.BindOutput(allCardpaymentsResult{}, "Receipt items list", nil),
	}

	(*mutation)["confirmCardpayment"] = &graphql.Field{
		Description: "Confirm a receipt item",
		Args:        util.BindArguments(confirmCardpaymentRequest{}),
		Resolve:     adminMutationConfirmCardpaymentResolver,
		Type:        util.BindOutput(cardpayment.ReceiptItem{}, "Receipt item information", nil),
	}

	(*query)["downloadInvoicesCsv"] = &graphql.Field{
		Description: "",
		Args:        util.BindArguments(cardpayment.DownloadInvoicesCsvRequest{}),
		Resolve:     adminQueryDownloadInvoicesCsvResolver,
		Type:        util.BindOutput(cardpayment.DownloadInvoicesCsvResponse{}, "Download invoices csv response", nil),
	}
}

func adminQueryGetCardpaymentResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}
	request := &cardpayment.ReceiptItemRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	resp, err := connections.CardPaymentService.GetItem(params.Context, request)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func adminQueryAllCardpaymentsResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}
	input := &allParams{}
	if err := util.UnmarshalArguments(params.Args, input); err != nil {
		return nil, err
	}
	req := &q.QueryParams{
		Filter: input.Filter,
		Sort:   input.Sort,
		Pagination: &q.QueryParamsPagination{
			Page:    input.Pagination.Page,
			PerPage: input.Pagination.PerPage,
		},
	}
	request := &cardpayment.ReceiptListFilterRequest{
		Params: &cardpayment.ReceiptListFilterRequest_ParamsValue{ParamsValue: req},
	}
	request.Admin = true
	resp, err := connections.CardPaymentService.ListItemsFilter(params.Context, request)
	if err != nil {
		return nil, err
	}
	return allCardpaymentsResult{
		Data:         resp.GetPayments(),
		Total:        resp.GetTotal(),
		NonConfirmed: resp.GetNonConfirmed(),
	}, nil
}

func adminMutationConfirmCardpaymentResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}
	req := &confirmCardpaymentRequest{}
	if err := util.UnmarshalArguments(params.Args, req); err != nil {
		return nil, err
	}
	user := params.Context.Value("user").(*auth.AuthResponse)
	request := &cardpayment.ReceiptItemConfirmRequest{
		ChargeId: req.ChargeId,
		Id:       req.Id,
		Admin:    user.GetAccount().GetIdValue(),
		Confirm:  req.Confirm,
	}
	resp, err := connections.CardPaymentService.ConfirmItem(params.Context, request)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func adminQueryDownloadInvoicesCsvResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}
	request := &cardpayment.DownloadInvoicesCsvRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	resp, err := connections.CardPaymentService.DownloadInvoicesCsv(params.Context, request)
	if err != nil {
		return nil, err
	}
	return resp, nil

}
