//go:generate go-bindata -pkg resolvers -ignore .*\.go .
package resolvers

import (
	"context"
	"encoding/json"
	"fmt"
	"github.com/eientei/wsgraphql/mutcontext"
	"github.com/golang/protobuf/ptypes"
	"github.com/graphql-go/graphql"
	"gitlab.okta-solutions.com/mashroom/backend/account/api"
	"gitlab.okta-solutions.com/mashroom/backend/auth/api"
	"gitlab.okta-solutions.com/mashroom/backend/common/generic/log"
	"gitlab.okta-solutions.com/mashroom/backend/common/graphql/util"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/connections"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/resolvers/errs"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/schema"
	"gitlab.okta-solutions.com/mashroom/backend/invitation/api"
	"io/ioutil"
	"net/http"
	"time"
)

type UserInfoForm struct {
	RefreshToken string `json:"refresh_token"`
	Email        string `json:"email"`
	Password     string `json:"password"`
	Phone        string `json:"phone"`
	FirstName    string `json:"first_name"`
	LastName     string `json:"last_name"`
}

type UserSignupForm struct {
	UserInfoForm
	Otp         string `json:"otp"`
	TosAccepted uint64 `json:"tos_accepted"`
}

type TosWordpress struct {
	Modified string
	Content  Content
}

type Content struct {
	Rendered string
}

type TosOutput struct {
	Text    string `json:"text"`
	Version int64  `json:"version"`
}

type OtpRequest struct {
	Phone string `json:"phone"`
}

type OtpResponse struct {
	Success bool `json:"success"`
}

type ResendRequest struct {
	Type invitation.ContactValidationType `json:"type"`
	Key  string                           `json:"key"`
}

type ResendResponse struct {
	Success bool `json:"success"`
}

type VerifyRequest struct {
	Password string `json:"password"`
	Otp      string `json:"otp"`
}

type VerifyResponse struct {
	PasswordOk bool `json:"password_ok"`
	OtpOk      bool `json:"otp_ok"`
}

var AnonymousRole = "<anonymous>"
var UserRole = ""
var AdminRole = "admin"

func init() {
	query := graphql.NewObject(graphql.ObjectConfig{
		Name:        "AuthQuery",
		Description: "Auth query root",
		Fields: graphql.Fields{
			"tos": &graphql.Field{
				Description: "Returns terms of service",
				Resolve:     AuthQueryTosResolver,
				Type:        util.BindOutput(TosOutput{}, "Tos text and version", nil),
			},
		},
	})

	schema.RootQueries["auth"] = &graphql.Field{
		Type: query,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return query, nil
		},
	}

	verify := graphql.NewObject(graphql.ObjectConfig{
		Name:        "AuthVerifyMutation",
		Description: "AuthVerify mutation root",
		Fields: graphql.Fields{
			"email": &graphql.Field{
				Description: "Verify email",
				Args:        util.BindArguments(auth.VerifyEmailRequest{}),
				Resolve:     AuthMutationVerifyEmailResolver,
				Type:        util.BindOutput(auth.VerifyEmailResponse{}, "Verify email status", nil),
			},
			"phone": &graphql.Field{
				Description: "Verify phone",
				Args:        util.BindArguments(auth.VerifyPhoneRequest{}),
				Resolve:     AuthMutationVerifyPhoneResolver,
				Type:        util.BindOutput(auth.VerifyPhoneResponse{}, "Verify phone status", nil),
			},
		},
	})

	mutation := graphql.NewObject(graphql.ObjectConfig{
		Name:        "AuthMutation",
		Description: "Auth mutation root",
		Fields: graphql.Fields{
			"login": &graphql.Field{
				Description: "Performs user login by username and password",
				Args:        util.BindArguments(auth.LoginRequest{}),
				Resolve:     AuthMutationLoginResolver,
				Type: util.BindOutput(auth.LoginResponse{}, "Login status and new access and refresh tokens", util.EnrichmentMap{
					"account": &util.Enrichment{
						Output: func(bind util.BindType) graphql.Output {
							return util.BindOutput(&account.Account{}, "", nil)
						},
						Resolver: func(p graphql.ResolveParams, oldvalue interface{}) (i interface{}, e error) {
							resp, ok := p.Source.(*auth.LoginResponse)
							if !ok || resp.GetAccessToken() == "" {
								return &account.Account{}, nil
							}
							ctx := p.Context.Value("globalctx").(mutcontext.MutableContext)
							authresp, err := connections.AuthService.Auth(p.Context, &auth.AuthRequest{
								AccessToken: resp.GetAccessToken(),
								AdminSite:   isAdminSite(ctx),
							})
							if err != nil {
								log.WithError(err).Error()
							}
							return authresp.GetAccount(), err
						},
					},
				}),
			},
			"refresh": &graphql.Field{
				Description: "Refreshes access token",
				Args:        util.BindArguments(auth.RefreshRequest{}),
				Resolve:     AuthMutationRefreshResolver,
				Type:        util.BindOutput(auth.RefreshResponse{}, "Refresh status and new access and refresh tokens", nil),
			},
			"logout": &graphql.Field{
				Description: "Logs out of current session",
				Resolve:     AuthMutationLogoutResolver,
				Type:        util.BindOutput(auth.LogoutResponse{}, "Logout status", nil),
			},
			"signup": &graphql.Field{
				Description: "Signs up for user account",
				Args:        util.BindArguments(auth.SignupRequest{}),
				Resolve:     AuthMutationSignupResolver,
				Type: util.BindOutput(auth.SignupResponse{}, "New access and refresh token and signup status", util.EnrichmentMap{
					"account": &util.Enrichment{
						Output: func(bind util.BindType) graphql.Output {
							return util.BindOutput(&account.Account{}, "", nil)
						},
						Resolver: func(p graphql.ResolveParams, oldvalue interface{}) (i interface{}, e error) {
							resp, ok := p.Source.(*auth.SignupResponse)
							if !ok || resp.GetAccessToken() == "" {
								return &account.Account{}, nil
							}
							authresp, err := connections.AuthService.Auth(p.Context, &auth.AuthRequest{
								AccessToken: resp.GetAccessToken(),
								AdminSite:   false,
							})
							if err != nil {
								log.WithError(err).Error()
							}
							return authresp.GetAccount(), err
						},
					},
				}),
			},
			"update": &graphql.Field{
				Description: "Updates user details",
				Args:        util.BindArguments(auth.UpdateRequest{}),
				Resolve:     AuthMutationUpdateResolver,
				Type:        util.BindOutput(auth.UpdateResponse{}, "Update status and new access and refresh tokens", nil),
			},
			"reset": &graphql.Field{
				Description: "Password reset",
				Args:        util.BindArguments(auth.ForgotPasswordRequest{}),
				Resolve:     AuthMutationResetPasswordResolver,
				Type:        util.BindOutput(auth.ForgotPasswordResponse{}, "Update password status", nil),
			},
			"resend": &graphql.Field{
				Description: "Resends OTP or email verification link challenge",
				Args:        util.BindArguments(ResendRequest{}),
				Resolve:     AuthMutationResendResolver,
				Type:        util.BindOutput(ResendResponse{}, "Result of resend operation", nil),
			},
			"verify": &graphql.Field{
				Type: verify,
				Resolve: func(p graphql.ResolveParams) (interface{}, error) {
					return verify, nil
				},
			},
		},
	})

	schema.RootMutations["auth"] = &graphql.Field{
		Type: mutation,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return mutation, nil
		},
	}
}

func isAdminSite(ctx context.Context) bool {
	if adminSite, ok := ctx.Value("adminSite").(bool); ok {
		return adminSite
	}
	return false
}

func HasRole(user *auth.AuthResponse, required string) bool {
	if user == nil {
		return required == AnonymousRole
	}
	if required == UserRole {
		return true
	}
	for _, role := range user.Roles {
		if role == required {
			return true
		}
	}
	return false
}

func CheckPermissions(queryContext context.Context, roles ...string) error {
	ctx := queryContext.Value("globalctx").(mutcontext.MutableContext)
	user := queryContext.Value("user").(*auth.AuthResponse)
	adminSite := isAdminSite(ctx)
	if adminSite {
		roles = append(roles, AdminRole)
	}

	any := false
	for _, r := range roles {
		if r == AdminRole && !adminSite {
			continue
		}
		any = any || HasRole(user, r)
	}

	if !any {
		return errs.AuthError
	}

	return nil
}

func AuthQueryTosResolver(params graphql.ResolveParams) (interface{}, error) {
	layout := "2006-01-02T15:04:05"
	//var tos int64
	ctx := params.Context.Value("globalctx").(mutcontext.MutableContext)
	//user := params.Context.Value("user").(*auth.AuthResponse)
	//if user == nil {
	//	return nil, errs.NoUserError
	//}
	//if user.Account.GetTosVersion() == nil {
	//	unixCreated := user.Account.GetCreatedAtValue().Seconds
	//	tos = unixCreated
	//} else {
	//	tos = user.Account.GetTosVersionValue()
	//}
	req, err := http.NewRequest("GET", "https://wordpress.staging.env.mashroom.com/wp-json/wp/v2/pages/?slug=terms-and-conditions", nil)
	if err != nil {
		return nil, err
	}
	client := http.Client{}
	resp, err := client.Do(req)

	defer resp.Body.Close()

	if err != nil {
		return nil, err
	}
	var tosWordpress []TosWordpress
	body, errRead := ioutil.ReadAll(resp.Body)
	if errRead != nil {
		return nil, errRead
	}
	err = json.Unmarshal(body, &tosWordpress)
	if err != nil {
		return nil, err
	}
	timeParse, _ := time.Parse(layout, tosWordpress[0].Modified)
	unix, _ := ptypes.TimestampProto(timeParse)
	//if tos > unix.Seconds {
	//	return &TosOutput{Text: "", Version: 0}, nil
	//}
	adminSite := isAdminSite(ctx)
	//bytes, err := Asset("ipsum.txt")
	//if err != nil {
	//	return nil, err
	//}
	prefix := ""
	if adminSite {
		prefix = "ADMIN: "
	}
	return &TosOutput{
		Text:    prefix + tosWordpress[0].Content.Rendered,
		Version: unix.Seconds,
	}, nil
}

func AuthMutationLoginResolver(params graphql.ResolveParams) (interface{}, error) {
	remote := params.Context.Value("remote").(string)
	request := &auth.LoginRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	ctx := params.Context.Value("globalctx").(mutcontext.MutableContext)
	adminSite := isAdminSite(ctx)
	request.Remote = &auth.LoginRequest_RemoteValue{RemoteValue: remote}
	request.AdminSite = &auth.LoginRequest_AdminSiteValue{AdminSiteValue: adminSite}
	resp, err := connections.AuthService.Login(params.Context, request)
	if err != nil || resp == nil || !resp.Success || len(resp.Errors) > 0 {
		return resp, err
	}
	authResp, err := connections.AuthService.Auth(ctx, &auth.AuthRequest{
		AccessToken: resp.GetAccessToken(),
		AdminSite:   adminSite,
	})
	if err != nil {
		return nil, err
	}
	ctx.Set("access_token", resp.GetAccessToken())
	ctx.Set("user", authResp)
	return resp, nil
}

func AuthMutationRefreshResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &auth.RefreshRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	if accessToken, ok := params.Context.Value("access_token").(string); ok {
		request.AccessToken = accessToken
	}
	ctx := params.Context.Value("globalctx").(mutcontext.MutableContext)
	adminSite := isAdminSite(ctx)
	request.AdminSite = adminSite
	resp, err := connections.AuthService.Refresh(params.Context, request)
	if err != nil || resp == nil || !resp.Success || len(resp.Errors) > 0 {
		return resp, err
	}
	authResp, err := connections.AuthService.Auth(ctx, &auth.AuthRequest{
		AccessToken: resp.GetAccessToken(),
		AdminSite:   adminSite,
	})
	if err != nil {
		return nil, err
	}
	ctx.Set("access_token", resp.GetAccessToken())
	ctx.Set("user", authResp)
	return resp, nil
}

func AuthMutationLogoutResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := CheckPermissions(params.Context, UserRole); err != nil {
		return nil, err
	}
	user := params.Context.Value("user").(*auth.AuthResponse)
	request := &auth.LogoutRequest{
		Session: user.Session,
	}
	if accessToken, ok := params.Context.Value("access_token").(string); ok {
		request.AccessToken = accessToken
	}
	ctx := params.Context.Value("globalctx").(mutcontext.MutableContext)
	request.AdminSite = isAdminSite(ctx)
	resp, err := connections.AuthService.Logout(params.Context, request)
	if err != nil || resp == nil || !resp.Success || len(resp.Errors) > 0 {
		return resp, err
	}
	ctx.Set("access_token", "")
	ctx.Set("user", (*auth.AuthResponse)(nil))
	return resp, nil
}

func AuthMutationSignupResolver(params graphql.ResolveParams) (interface{}, error) {
	ctx := params.Context.Value("globalctx").(mutcontext.MutableContext)
	adminSite := isAdminSite(ctx)
	// CheckPermissions doesn't cover this case
	if adminSite {
		return nil, errs.WrongQueryError
	}

	request := &auth.SignupRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.Account.Admin = &account.Account_AdminValue{AdminValue: false}
	remote := params.Context.Value("remote").(string)
	request.Remote = &auth.SignupRequest_RemoteValue{RemoteValue: remote}
	resp, err := connections.AuthService.Signup(params.Context, request)
	if err != nil || resp == nil || !resp.Success || len(resp.Errors) > 0 {
		return resp, err
	}
	authResp, err := connections.AuthService.Auth(ctx, &auth.AuthRequest{
		AccessToken: resp.GetAccessToken(),
		AdminSite:   false,
	})
	if err != nil {
		return nil, err
	}
	ctx.Set("access_token", resp.GetAccessToken())
	ctx.Set("user", authResp)
	return resp, nil
}

func AuthMutationUpdateResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := CheckPermissions(params.Context, UserRole); err != nil {
		return nil, err
	}
	request := &auth.UpdateRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	user := params.Context.Value("user").(*auth.AuthResponse)
	request.GetAccount().Id = user.GetAccount().GetId()
	if accessToken, ok := params.Context.Value("access_token").(string); ok {
		fmt.Println(accessToken)
		request.AccessToken = accessToken
	}
	resp, err := connections.AuthService.Update(params.Context, request)
	if err != nil || resp == nil || !resp.Success || len(resp.Errors) > 0 {
		return resp, err
	}
	ctx := params.Context.Value("globalctx").(mutcontext.MutableContext)
	authResp, err := connections.AuthService.Auth(ctx, &auth.AuthRequest{
		AccessToken: resp.GetAccessToken(),
		AdminSite:   isAdminSite(ctx),
	})
	if err != nil {
		return nil, err
	}
	ctx.Set("access_token", resp.GetAccessToken())
	ctx.Set("user", authResp)
	return resp, nil
}

func AuthMutationResendResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &ResendRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	user := params.Context.Value("user").(*auth.AuthResponse)
	userid := "newuser"
	if user != nil {
		userid = user.GetAccount().GetIdValue()
	}
	resp, err := connections.InvitationService.CreateContactValidation(params.Context, &invitation.CreateContactValidationRequest{
		Type:      request.Type,
		AccountId: userid,
		Key:       request.Key,
		Force:     true,
	})
	if err != nil {
		return nil, err
	}
	return &ResendResponse{Success: resp.GetSuccess()}, nil
}

func AuthMutationVerifyEmailResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := CheckPermissions(params.Context, UserRole); err != nil {
		return nil, err
	}
	request := &auth.VerifyEmailRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	if accessToken, ok := params.Context.Value("access_token").(string); ok {
		request.AccessToken = accessToken
	}
	return connections.AuthService.VerifyEmail(params.Context, request)
}

func AuthMutationVerifyPhoneResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := CheckPermissions(params.Context, UserRole); err != nil {
		return nil, err
	}
	request := &auth.VerifyPhoneRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	if accessToken, ok := params.Context.Value("access_token").(string); ok {
		request.AccessToken = accessToken
	}
	return connections.AuthService.VerifyPhone(params.Context, request)
}

func AuthMutationResetPasswordResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &auth.ForgotPasswordRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	if remote, ok := params.Context.Value("remote").(string); ok {
		request.Remote = &auth.ForgotPasswordRequest_RemoteValue{RemoteValue: remote}
	}
	return connections.AuthService.ForgotPassword(params.Context, request)
}
