package resolvers

import (
	"github.com/graphql-go/graphql"
	"gitlab.okta-solutions.com/mashroom/backend/auth/api"
	"gitlab.okta-solutions.com/mashroom/backend/common/graphql/util"
	"gitlab.okta-solutions.com/mashroom/backend/contract/api"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/connections"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/resolvers/errs"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/schema"
)

var ContentManagerRole = "content-manager"

func init() {
	renderArguments := util.BindArguments(contract.RenderRequest{})
	renderArguments["values_json"] = &graphql.ArgumentConfig{
		Type: graphql.String,
	}

	query := graphql.NewObject(graphql.ObjectConfig{
		Name:        "ContractQuery",
		Description: "Contract query root",
		Fields: graphql.Fields{
			"history": &graphql.Field{
				Description: "Returns history of given contract at given depth",
				Args:        util.BindArguments(contract.HistoryRequest{}),
				Resolve:     ContractQueryHistoryResolver,
				Type:        util.BindOutput(contract.HistoryResponse{}, "Revisions list", nil),
			},
			"list": &graphql.Field{
				Description: "Returns list of children contracts",
				Args:        util.BindArguments(contract.ListRequest{}),
				Resolve:     ContractQueryListResolver,
				Type:        util.BindOutput(contract.ListResponse{}, "Contracts list", nil),
			},
			"render": &graphql.Field{
				Description: "Renders given contract with specified format and values provided",
				Args:        renderArguments,
				Resolve:     ContractQueryRenderResolver,
				Type:        graphql.String,
			},
		},
	})

	schema.RootQueries["contract"] = &graphql.Field{
		Type: query,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return query, nil
		},
	}

	mutation := graphql.NewObject(graphql.ObjectConfig{
		Name:        "ContractMutation",
		Description: "Contract mutation root",
		Fields: graphql.Fields{
			"create": &graphql.Field{
				Description: "Creates new contract",
				Args:        util.BindArguments(contract.CreateRequest{}),
				Resolve:     ContractMutationCreateResolver,
				Type:        util.BindOutput(contract.CreateResponse{}, "Create status", nil),
			},
			"update": &graphql.Field{
				Description: "Updates existing contract",
				Args:        util.BindArguments(contract.UpdateRequest{}),
				Resolve:     ContractMutationUpdateResolver,
				Type:        util.BindOutput(contract.UpdateResponse{}, "Update status", nil),
			},
			"edit": &graphql.Field{
				Description: "Updates revision of a contract",
				Args:        util.BindArguments(contract.EditRequest{}),
				Resolve:     ContractMutationEditResolver,
				Type:        util.BindOutput(contract.EditResponse{}, "Edit status", nil),
			},
			"delete": &graphql.Field{
				Description: "Deletes contract",
				Args:        util.BindArguments(contract.DeleteRequest{}),
				Resolve:     ContractMutationDeleteResolver,
				Type:        util.BindOutput(contract.DeleteResponse{}, "Delete status", nil),
			},
		},
	})

	schema.RootMutations["contract"] = &graphql.Field{
		Type: mutation,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return mutation, nil
		},
	}
}

func ContractQueryHistoryResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	if !HasRole(user, ContentManagerRole) {
		return nil, errs.AuthError
	}
	request := &contract.HistoryRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.ContractService.History(params.Context, request)
}

func ContractQueryListResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	if !HasRole(user, ContentManagerRole) {
		return nil, errs.AuthError
	}
	request := &contract.ListRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.ContractService.List(params.Context, request)
}

func ContractQueryRenderResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	if !HasRole(user, ContentManagerRole) {
		return nil, errs.AuthError
	}
	values := params.Args["values_json"]
	if values == nil {
		return nil, errs.NoValuesError
	}
	params.Args["values_json"] = nil
	request := &contract.RenderRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.ValuesJson = []byte(values.(string))
	res, err := connections.ContractService.Render(params.Context, request)
	if err != nil {
		return nil, err
	}
	return res.Fileid, nil
}

func ContractMutationCreateResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	if !HasRole(user, ContentManagerRole) {
		return nil, errs.AuthError
	}
	request := &contract.CreateRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.ContractService.Create(params.Context, request)
}

func ContractMutationUpdateResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	if !HasRole(user, ContentManagerRole) {
		return nil, errs.AuthError
	}
	request := &contract.UpdateRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.ContractService.Update(params.Context, request)
}

func ContractMutationEditResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	if !HasRole(user, ContentManagerRole) {
		return nil, errs.AuthError
	}
	request := &contract.EditRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.ContractService.Edit(params.Context, request)
}

func ContractMutationDeleteResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	if !HasRole(user, ContentManagerRole) {
		return nil, errs.AuthError
	}
	request := &contract.DeleteRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.ContractService.Delete(params.Context, request)
}
