package resolvers

import (
	"github.com/graphql-go/graphql"
	account "gitlab.okta-solutions.com/mashroom/backend/account/api"
	"gitlab.okta-solutions.com/mashroom/backend/auth/api"
	"gitlab.okta-solutions.com/mashroom/backend/common/graphql/util"
	"gitlab.okta-solutions.com/mashroom/backend/document/api"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/connections"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/resolvers/errs"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/schema"
	offer "gitlab.okta-solutions.com/mashroom/backend/offer/api"
	"gitlab.okta-solutions.com/mashroom/backend/property/api"
)

type deleteRequest struct {
	Id string `json:"id,omitempty"`
}

func init() {
	query := graphql.NewObject(graphql.ObjectConfig{
		Name:        "DocumentQuery",
		Description: "Document query root",
		Fields: graphql.Fields{
			"get": &graphql.Field{
				Description: "Get document by id",
				Args:        util.BindArguments(document.GetRequest{}),
				Resolve:     DocumentQueryGetResolver,
				Type:        util.BindOutput(document.GetResponse{}, "Document with given id", nil),
			},
			"list": &graphql.Field{
				Description: "Get documents by filters",
				Args:        util.BindArguments(document.ListRequest{}),
				Resolve:     DocumentQueryListResolver,
				Type:        util.BindOutput(document.ListResponse{}, "Document list", nil),
			},
		},
	})

	schema.RootQueries["document"] = &graphql.Field{
		Type: query,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return query, nil
		},
	}

	mutation := graphql.NewObject(graphql.ObjectConfig{
		Name:        "DocumentMutation",
		Description: "Document mutation root",
		Fields: graphql.Fields{
			"create": &graphql.Field{
				Description: "Create document",
				Args:        util.BindArguments(document.CreateRequest{}),
				Resolve:     DocumentMutationCreateResolver,
				Type:        util.BindOutput(document.CreateResponse{}, "Document create response", nil),
			},
			"delete": &graphql.Field{
				Description: "Delete document",
				Args:        util.BindArguments(deleteRequest{}),
				Resolve:     DocumentMutationDeleteResolver,
				Type:        util.BindOutput(document.DeleteResponse{}, "Document delete response", nil),
			},
		},
	})

	schema.RootMutations["document"] = &graphql.Field{
		Type: mutation,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return mutation, nil
		},
	}
}

func DocumentQueryGetResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &document.GetRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.DocumentService.Get(params.Context, request)
}

func DocumentQueryListResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &document.ListRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	if request.GetProperty() != "" {
		resp, err := connections.PropertyService.Get(params.Context, &property.GetRequest{
			Id: request.GetProperty(),
		})
		if err == nil {
			request.User = resp.GetProperty().GetPeopleValue().GetLandlordValue()
		}
	}

	return connections.DocumentService.List(params.Context, request)
}

func DocumentMutationCreateResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &document.CreateRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()

	return connections.DocumentService.Create(params.Context, request)
}

func DocumentMutationDeleteResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	req := &deleteRequest{}
	if err := util.UnmarshalArguments(params.Args, req); err != nil {
		return nil, err
	}
	userId := user.GetAccount().GetIdValue()

	// Delete document from account
	reqAcc := &account.DeleteDocumentRequest{
		Id:        req.Id,
		User:      userId,
		AdminSite: false,
	}
	if resp, err := connections.AccountService.DeleteDocument(params.Context, reqAcc); (resp != nil) && (resp.GetError()) {
		return nil, err
	}

	// Delete document from property
	reqProp := &property.DeleteDocumentRequest{
		Id:        req.Id,
		User:      userId,
		AdminSite: false,
	}
	if resp, err := connections.PropertyService.DeleteDocument(params.Context, reqProp); (resp != nil) && (resp.GetError()) {
		return nil, err
	}

	// Delete document from offer
	reqOffer := &offer.DeleteDocumentRequest{
		Id:        req.Id,
		User:      userId,
		AdminSite: false,
	}
	if resp, err := connections.OfferService.DeleteDocument(params.Context, reqOffer); (resp != nil) && (resp.GetError()) {
		return nil, err
	}

	// Delete document itself
	reqDoc := &document.DeleteRequest{
		Id:        req.Id,
		User:      userId,
		AdminSite: false,
	}
	return connections.DocumentService.Delete(params.Context, reqDoc)
}
