package resolvers

import (
	"github.com/graphql-go/graphql"
	"gitlab.okta-solutions.com/mashroom/backend/account/api"
	"gitlab.okta-solutions.com/mashroom/backend/auth/api"
	"gitlab.okta-solutions.com/mashroom/backend/common/graphql/util"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/connections"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/resolvers/errs"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/schema"
	"gitlab.okta-solutions.com/mashroom/backend/invitation/api"
	"gitlab.okta-solutions.com/mashroom/backend/property/api"
)

func init() {
	query := graphql.NewObject(graphql.ObjectConfig{
		Name:        "InvitationQuery",
		Description: "Invitation query root",
		Fields: graphql.Fields{
			"incoming": &graphql.Field{
				Description: "Get incoming invitations",
				Resolve:     InvitationIncomingQueryResolver,
				Type:        util.BindOutput(invitation.InvitationListResponse{}, "Invitations list", nil),
			},
			"outgoing": &graphql.Field{
				Description: "Get outgoing invitations",
				Resolve:     InvitationOutgoingQueryResolver,
				Type:        util.BindOutput(invitation.InvitationListResponse{}, "Invitations list", nil),
			},
		},
	})

	schema.RootQueries["invitation"] = &graphql.Field{
		Type: query,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return query, nil
		},
	}

	mutation := graphql.NewObject(graphql.ObjectConfig{
		Name:        "InvitationMutation",
		Description: "Invitation mutation root",
		Fields: graphql.Fields{
			"create": &graphql.Field{
				Description: "Create invitation",
				Args:        util.BindArguments(invitation.InvitationInfo{}),
				Resolve:     InvitationCreateMutationResolver,
				Type:        util.BindOutput(invitation.CreateInvitationResponse{}, "Create status", nil),
			},
			"accept": &graphql.Field{
				Description: "Accept invitation",
				Args:        util.BindArguments(invitation.InvitationActionRequest{}),
				Resolve:     InvitationAcceptMutationResolver,
				Type:        util.BindOutput(invitation.InvitationActionResponse{}, "Accept status", nil),
			},
			"reject": &graphql.Field{
				Description: "Reject invitation",
				Args:        util.BindArguments(invitation.InvitationActionRequest{}),
				Resolve:     InvitationRejectMutationResolver,
				Type:        util.BindOutput(invitation.InvitationActionResponse{}, "Reject status", nil),
			},
			"decline": &graphql.Field{
				Description: "Decline invitation by id, no auth",
				Args:        util.BindArguments(invitation.InvitationActionRequest{}),
				Resolve:     InvitationDeclineByLinkMutationResolver,
				Type:        util.BindOutput(invitation.InvitationActionResponse{}, "Decline status", nil),
			},
		},
	})

	schema.RootMutations["invitation"] = &graphql.Field{
		Type: mutation,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return mutation, nil
		},
	}
}

func InvitationIncomingQueryResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &invitation.InvitationListIncomingRequest{
		Email: user.GetAccount().GetContactValue().GetEmailPrimaryValue(),
	}
	return connections.InvitationService.InvitationListIncoming(params.Context, request)
}

func InvitationOutgoingQueryResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &invitation.InvitationListOutgoingRequest{
		User: user.GetAccount().GetIdValue(),
	}
	return connections.InvitationService.InvitationListOutgoing(params.Context, request)
}

func InvitationCreateMutationResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &invitation.InvitationInfo{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	prop, err := connections.PropertyService.Get(params.Context, &property.GetRequest{
		Id: request.GetPropertyId(),
	})
	if err != nil {
		return nil, err
	}

	inviter, err := connections.AccountService.Get(params.Context, &account.GetRequest{
		Id: user.GetAccount().GetIdValue(),
	})
	if err != nil {
		return nil, err
	}

	email := request.GetInviteeEmail()
	phone := request.GetInviteePhone()
	firstName := request.GetInviteeFirstName()
	lastName := request.GetInviteeLastName()
	message := request.GetMessage()
	if message == "" {
		message = request.Body
	}

	_, err = connections.PropertyService.Invite(params.Context, &property.InviteRequest{
		Property:         prop.GetProperty().GetIdValue(),
		Type:             request.GetType().String(),
		InviterId:        inviter.GetAccount().GetIdValue(),
		InviteeId:        request.GetInviteeId(),
		InviteeEmail:     email,
		InviteePhone:     phone,
		InviteeFirstName: firstName,
		InviteeLastName:  lastName,
		SiteUrl:          SiteUrl,
		Message:          message,
	})
	if err != nil {
		return nil, err
	}
	return &invitation.CreateInvitationResponse{
		Type: request.GetType(),
	}, nil
}

func InvitationAcceptMutationResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &invitation.InvitationActionRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.Email = user.GetAccount().GetContactValue().GetEmailPrimaryValue()
	request.User = user.GetAccount().GetIdValue()
	return connections.InvitationService.AcceptInvitation(params.Context, request)
}

func InvitationRejectMutationResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &invitation.InvitationActionRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.Email = user.GetAccount().GetContactValue().GetEmailPrimaryValue()
	request.User = user.GetAccount().GetIdValue()
	return connections.InvitationService.DeclineInvitation(params.Context, request)
}

func InvitationDeclineByLinkMutationResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &invitation.InvitationActionRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	return connections.InvitationService.DeclineInvitationById(params.Context, request)
}
