package resolvers

import (
	"github.com/graphql-go/graphql"
	"gitlab.okta-solutions.com/mashroom/backend/account/api"
	"gitlab.okta-solutions.com/mashroom/backend/auth/api"
	"gitlab.okta-solutions.com/mashroom/backend/common/generic/log"
	"gitlab.okta-solutions.com/mashroom/backend/common/graphql/util"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/connections"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/schema"
	"gitlab.okta-solutions.com/mashroom/backend/oauth/api"
)

type OauthAuthSignup struct {
	Token oauth.TokenAccessRequest `json:"token"`
	Info  auth.SignupRequest       `json:"info"`
}

type OauthAuthLogin struct {
	Token oauth.TokenAccessRequest `json:"token"`
	Info  auth.OauthRequest        `json:"info"`
}

func init() {
	query := graphql.NewObject(graphql.ObjectConfig{
		Name:        "OauthQuery",
		Description: "Oauth query root",
		Fields: graphql.Fields{
			"facebookUserInfoWithToken": &graphql.Field{
				Description: "Get facebook user info using access token",
				Args:        util.BindArguments(oauth.TokenAccessRequest{}),
				Resolve:     OauthFacebookUserInfoWithTokenResolver,
				Type:        util.BindOutput(oauth.FacebookGetUserInfoResponse{}, "Facebook user info", nil),
			},
		},
	})

	schema.RootQueries["oauth"] = &graphql.Field{
		Type: query,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return query, nil
		},
	}

	mutation := graphql.NewObject(graphql.ObjectConfig{
		Name:        "OauthMutation",
		Description: "Oauth mutation root",
		Fields: graphql.Fields{
			"facebookLogin": &graphql.Field{
				Description: "Login using facebook token",
				Args:        util.BindArguments(OauthAuthLogin{}),
				Resolve:     OauthFacebookLogin,
				Type: util.BindOutput(auth.OauthResponse{}, "Facebook login", util.EnrichmentMap{
					"account": &util.Enrichment{
						Output: func(bind util.BindType) graphql.Output {
							return util.BindOutput(&account.Account{}, "", nil)
						},
						Resolver: func(p graphql.ResolveParams, oldvalue interface{}) (i interface{}, e error) {
							resp, ok := p.Source.(*auth.OauthResponse)
							if !ok || resp.GetAccessToken() == "" {
								return &account.Account{}, nil
							}
							authresp, err := connections.AuthService.Auth(p.Context, &auth.AuthRequest{
								AccessToken: resp.GetAccessToken(),
								AdminSite:   false,
							})
							if err != nil {
								log.WithError(err).Error()
							}
							return authresp.GetAccount(), err
						},
					},
				}),
			},
			"facebookSignup": &graphql.Field{
				Description: "Signup using facebook token",
				Args:        util.BindArguments(OauthAuthSignup{}),
				Resolve:     OauthFacebookSignup,
				Type:        util.BindOutput(auth.SignupResponse{}, "Facebook signup", nil),
			},
		},
	})

	schema.RootMutations["oauth"] = &graphql.Field{
		Type: mutation,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return mutation, nil
		},
	}
}

func OauthFacebookUserInfoWithTokenResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &oauth.TokenAccessRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.OauthService.FacebookGetUserInfoWithToken(params.Context, request)
}

func OauthFacebookSignup(params graphql.ResolveParams) (interface{}, error) {
	log.Infof("signup %+v", params.Args)
	request := &OauthAuthSignup{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	log.Infof("signup2 %+v", connections.OauthService)
	resp, err := connections.OauthService.FacebookGetUserInfoWithToken(params.Context, &request.Token)
	log.Infof("signup3 %+v %+v", resp, err)
	if err != nil {
		return nil, err
	}
	remote := params.Context.Value("remote").(string)
	log.Infof("signup4 %+v", remote)

	if request.Info.Account == nil {
		request.Info.Account = &account.Account{}
	}

	if request.Info.GetAccount().Contact == nil {
		request.Info.Account.Contact = &account.Account_ContactValue{ContactValue: &account.AccountContact{}}
	}

	if request.Info.GetAccount().Details == nil {
		request.Info.Account.Details = &account.Account_DetailsValue{DetailsValue: &account.AccountDetails{}}
	}

	request.Info.GetAccount().Admin = &account.Account_AdminValue{AdminValue: false}

	if len(request.Info.GetAccount().GetContactValue().GetEmailPrimaryValue()) == 0 {
		request.Info.GetAccount().GetContactValue().EmailPrimary = &account.AccountContact_EmailPrimaryValue{EmailPrimaryValue: resp.GetEmail()}
	}
	if len(request.Info.GetAccount().GetDetailsValue().GetFirstNameValue()) == 0 {
		request.Info.GetAccount().GetDetailsValue().FirstName = &account.AccountDetails_FirstNameValue{FirstNameValue: resp.GetFirstName()}
	}
	if len(request.Info.GetAccount().GetDetailsValue().GetLastNameValue()) == 0 {
		request.Info.GetAccount().GetDetailsValue().LastName = &account.AccountDetails_LastNameValue{LastNameValue: resp.GetLastName()}
	}
	request.Info.Remote = &auth.SignupRequest_RemoteValue{RemoteValue: remote}
	request.Info.OauthId = &auth.SignupRequest_OauthIdValue{OauthIdValue: resp.GetId()}
	request.Info.OauthType = &auth.SignupRequest_OauthTypeValue{OauthTypeValue: "facebook"}
	log.Infof("signup5 %+v", request.Info)
	return connections.AuthService.Signup(params.Context, &request.Info)
}

func OauthFacebookLogin(params graphql.ResolveParams) (interface{}, error) {
	request := &OauthAuthLogin{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	log.Infof("oauth, %+v", connections.OauthService)
	resp, err := connections.OauthService.FacebookGetUserInfoWithToken(params.Context, &request.Token)
	log.Infof("%+v - %+v", resp, err)
	if err != nil {
		return nil, err
	}
	remote := params.Context.Value("remote").(string)
	log.Info("remote")

	request.Info.Email = &auth.OauthRequest_EmailValue{EmailValue: resp.GetEmail()}
	request.Info.Remote = &auth.OauthRequest_RemoteValue{RemoteValue: remote}
	request.Info.OauthId = &auth.OauthRequest_OauthIdValue{OauthIdValue: resp.GetId()}
	request.Info.OauthType = &auth.OauthRequest_OauthTypeValue{OauthTypeValue: "facebook"}

	log.Infof("%+v", request.Info)

	return connections.AuthService.OauthLogin(params.Context, &request.Info)
}
