package resolvers

import (
	"context"

	"github.com/graphql-go/graphql"
	account "gitlab.okta-solutions.com/mashroom/backend/account/api"
	auth "gitlab.okta-solutions.com/mashroom/backend/auth/api"
	"gitlab.okta-solutions.com/mashroom/backend/common/graphql/util"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/connections"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/resolvers/errs"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/schema"
	property "gitlab.okta-solutions.com/mashroom/backend/property/api"
)

func CensoredAccount(acc *account.Account) *account.Account {
	return &account.Account{
		Id:        acc.GetId(),
		CreatedAt: acc.GetCreatedAt(),
		Details:   acc.GetDetails(),
		Contact:   acc.GetContact(),
	}
}

func init() {
	query := graphql.NewObject(graphql.ObjectConfig{
		Name:        "PropertyQuery",
		Description: "Property query root",
		Fields: graphql.Fields{
			"get": &graphql.Field{
				Description: "Retrieves specified property",
				Args:        util.BindArguments(property.GetRequest{}),
				Resolve:     PropertyQueryGetResolver,
				Type:        util.BindOutput(property.GetResponse{}, "Property information", nil),
			},
			"my": &graphql.Field{
				Description: "Retrieves owned property",
				Args:        util.BindArguments(property.PropertyListRequest{}),
				Resolve:     PropertyQueryMyResolver,
				Type:        util.BindOutput(property.PropertyListResponse{}, "Property list information", nil),
			},
			"list": &graphql.Field{
				Description: "Retrieves owned property",
				Args:        util.BindArguments(property.PropertyListRequest{}),
				Resolve:     PropertyQueryListResolver,
				Type:        util.BindOutput(property.PropertyListResponse{}, "Property list information", nil),
			},
			"search": &graphql.Field{
				Description: "Searches property by provided criteria",
				Args:        util.BindArguments(property.PropertySearchRequest{}),
				Resolve:     PropertyQuerySearchResolver,
				Type:        util.BindOutput(property.PropertySearchResponse{}, "Property list information with pagination", nil),
			},
			"searchinpolygon": &graphql.Field{
				Description: "Searches property within a geographical polygon",
				Args:        util.BindArguments(property.PropertySearchInPolygonRequest{}),
				Resolve:     PropertyQuerySearchInPolygonResolver,
				Type:        util.BindOutput(property.PropertySearchResponse{}, "Property list information with pagination", nil),
			},
			"listview": &graphql.Field{
				Description: "Lists view requests for property",
				Args:        util.BindArguments(property.ListViewRequest{}),
				Resolve:     PropertyQueryListViewResolver,
				Type:        util.BindOutput(property.ListViewResponse{}, "Property view listing", nil),
			},
			"viewreserved": &graphql.Field{
				Description: "Reserved view intervals",
				Args:        util.BindArguments(property.ViewReservedRequest{}),
				Resolve:     PropertyQueryViewReservedResolver,
				Type:        util.BindOutput(property.ViewReservedResponse{}, "Reserved intervals", nil),
			},
			"renderhei": &graphql.Field{
				Description: "Render Home Emergency Insurance documents",
				Args:        util.BindArguments(property.RenderHeiDocumentsRequest{}),
				Resolve:     PropertyQueryRenderHeiDocuments,
				Type:        util.BindOutput(property.RenderHeiDocumentsResponse{}, "Hei documents", nil),
			},
			"renderrgi": &graphql.Field{
				Description: "Render Rent Guarantee Insurance documents",
				Args:        util.BindArguments(property.RenderRgiDocumentsRequest{}),
				Resolve:     PropertyQueryRenderRgiDocuments,
				Type:        util.BindOutput(property.RenderRgiDocumentsResponse{}, "Rgi documents", nil),
			},
		},
	})

	schema.RootQueries["property"] = &graphql.Field{
		Type: query,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return query, nil
		},
	}

	mutation := graphql.NewObject(graphql.ObjectConfig{
		Name:        "PropertyMutation",
		Description: "Property mutation root",
		Fields: graphql.Fields{
			"create": &graphql.Field{
				Description: "Creates new property",
				Args:        util.BindArguments(property.CreateRequest{}),
				Resolve:     PropertyMutationCreateResolver,
				Type:        util.BindOutput(property.CreateResponse{}, "Create status", nil),
			},
			"update": &graphql.Field{
				Description: "Updates existing property",
				Args:        util.BindArguments(property.UpdateRequest{}),
				Resolve:     PropertyMutationUpdateResolver,
				Type:        util.BindOutput(property.UpdateResponse{}, "Create status", nil),
			},
			"delete": &graphql.Field{
				Description: "Deletes existing property",
				Args:        util.BindArguments(property.DeleteRequest{}),
				Resolve:     PropertyMutationDeleteResolver,
				Type:        util.BindOutput(property.DeleteResponse{}, "Delete status", nil),
			},
			"restore": &graphql.Field{
				Description: "Restore property",
				Args:        util.BindArguments(property.RestoreRequest{}),
				Resolve:     PropertyMutationRestoreResolver,
				Type:        util.BindOutput(property.RestoreResponse{}, "Restore status", nil),
			},
			"publish": &graphql.Field{
				Description: "Publish listing request",
				Args:        util.BindArguments(property.PublishRequest{}),
				Resolve:     PropertyMutationPublishResolver,
				Type:        util.BindOutput(property.PublishResponse{}, "Create listing status", nil),
			},
			"publishconsent": &graphql.Field{
				Description: "Publish listing consent request",
				Args:        util.BindArguments(property.PublishConsentRequest{}),
				Resolve:     PropertyMutationPublishConsentResolver,
				Type:        util.BindOutput(property.PublishConsentResponse{}, "Consent listing status", nil),
			},
			"unpublish": &graphql.Field{
				Description: "Unpublish listing request",
				Args:        util.BindArguments(property.UnpublishRequest{}),
				Resolve:     PropertyMutationUnpublishResolver,
				Type:        util.BindOutput(property.UnpublishResponse{}, "Drop listing status", nil),
			},
			"claim": &graphql.Field{
				Description: "Claim property",
				Args:        util.BindArguments(property.ClaimRequest{}),
				Resolve:     PropertyMutationClaimResolver,
				Type:        util.BindOutput(property.ClaimResponse{}, "Claim status", nil),
			},
			"commitstate": &graphql.Field{
				Description: "Commit state",
				Args:        util.BindArguments(property.CommitStateRequest{}),
				Resolve:     PropertyMutationCommitstateResolver,
				Type:        util.BindOutput(property.CommitStateResponse{}, "Commit status", nil),
			},
			"requestview": &graphql.Field{
				Description: "Request property view",
				Args:        util.BindArguments(property.RequestViewRequest{}),
				Resolve:     PropertyMutationRequestViewResolver,
				Type:        util.BindOutput(property.RequestViewResponse{}, "Request view status", nil),
			},
			"cancelview": &graphql.Field{
				Description: "Cancel property view",
				Args:        util.BindArguments(property.CancelViewRequest{}),
				Resolve:     PropertyMutationCancelViewResolver,
				Type:        util.BindOutput(property.CancelViewResponse{}, "Cancel view status", nil),
			},
			"acceptview": &graphql.Field{
				Description: "Accept property view",
				Args:        util.BindArguments(property.AcceptViewRequest{}),
				Resolve:     PropertyMutationAcceptViewResolver,
				Type:        util.BindOutput(property.AcceptViewResponse{}, "Accept view status", nil),
			},
			"rejectview": &graphql.Field{
				Description: "Reject property view",
				Args:        util.BindArguments(property.RejectViewRequest{}),
				Resolve:     PropertyMutationRejectViewResolver,
				Type:        util.BindOutput(property.RejectViewResponse{}, "Reject view status", nil),
			},
			"completeview": &graphql.Field{
				Description: "Complete property view",
				Args:        util.BindArguments(property.CompleteViewRequest{}),
				Resolve:     PropertyMutationCompleteViewResolver,
				Type:        util.BindOutput(property.CompleteViewResponse{}, "Complete view status", nil),
			},
			"failview": &graphql.Field{
				Description: "Fail property view",
				Args:        util.BindArguments(property.FailViewRequest{}),
				Resolve:     PropertyMutationFailViewResolver,
				Type:        util.BindOutput(property.FailViewResponse{}, "Fail view status", nil),
			},
			"changebankdetails": &graphql.Field{
				Name:    "Change bank details",
				Args:    util.BindArguments(property.ChangeBankDetailsRequest{}),
				Resolve: PropertyMutationChangeBankDetails,
				Type:    util.BindOutput(property.ChangeBankDetailsResponse{}, "Change bank details status", nil),
			},
			"withdraw": &graphql.Field{
				Description: "Withdraw property",
				Args:        util.BindArguments(property.WithdrawnRequest{}),
				Resolve:     adminMutationWithdrawPropertyResolver,
				Type:        util.BindOutput(property.WithdrawnResponse{}, "Withdraw property response", nil),
			},
		},
	})

	schema.RootMutations["property"] = &graphql.Field{
		Type: mutation,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return mutation, nil
		},
	}
}

func ForeignProperty(user *auth.AuthResponse, prop *property.PropertyPeople) bool {
	return user == nil || (prop.GetLandlordValue() != user.GetAccount().GetIdValue() && prop.GetSubmitterValue() != user.GetAccount().GetIdValue() && prop.GetManagerValue() != user.GetAccount().GetIdValue())
}

func PropertyQueryGetResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	request := &property.GetRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	if user != nil {
		request.User = user.GetAccount().GetIdValue()
	}
	resp, err := connections.PropertyService.Get(params.Context, request)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func PropertyQueryMyResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &property.PropertyListRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = &property.PropertyListRequest_UserValue{UserValue: user.GetAccount().GetIdValue()}
	resp, err := connections.PropertyService.List(params.Context, request)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func PropertyQueryListResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &property.PropertyListRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = &property.PropertyListRequest_UserValue{UserValue: user.GetAccount().GetIdValue()}
	resp, err := connections.PropertyService.List(params.Context, request)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func PropertyQuerySearchResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &property.PropertySearchRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	resp, err := connections.PropertyService.Search(params.Context, request)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func PropertyQuerySearchInPolygonResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &property.PropertySearchInPolygonRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	resp, err := connections.PropertyService.SearchInPolygon(params.Context, request)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func PropertyQueryViewReservedResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &property.ViewReservedRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	resp, err := connections.PropertyService.ViewReserved(params.Context, request)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func PropertyMutationCreateResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}

	request := &property.CreateRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()
	resp, err := connections.PropertyService.Create(params.Context, request)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func PropertyMutationUpdateResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}

	request := &property.UpdateRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()
	resp, err := connections.PropertyService.Update(params.Context, request)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func PropertyMutationDeleteResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}

	request := &property.DeleteRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	request.User = user.GetAccount().GetIdValue()
	return connections.PropertyService.Delete(params.Context, request)
}

func PropertyMutationRestoreResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}

	request := &property.RestoreRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()
	return connections.PropertyService.Restore(params.Context, request)
}

func PropertyMutationPublishResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &property.PublishRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()
	return connections.PropertyService.Publish(params.Context, request)
}

func PropertyMutationPublishConsentResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &property.PublishConsentRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()
	return connections.PropertyService.PublishConsent(params.Context, request)
}

func PropertyMutationUnpublishResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &property.UnpublishRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()
	return connections.PropertyService.Unpublish(params.Context, request)
}

func PropertyMutationClaimResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &property.ClaimRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()
	return connections.PropertyService.Claim(params.Context, request)
}

func PropertyMutationCommitstateResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &property.CommitStateRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()
	return connections.PropertyService.CommitState(params.Context, request)
}

func PropertyQueryListViewResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &property.ListViewRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()
	resp, err := connections.PropertyService.ListView(params.Context, request)
	if err != nil {
		return nil, err
	}
	return resp, err
}

func PropertyMutationRequestViewResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &property.RequestViewRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()
	return connections.PropertyService.RequestView(context.Background(), request)
}

func PropertyMutationCancelViewResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &property.CancelViewRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()
	return connections.PropertyService.CancelView(context.Background(), request)
}

func PropertyMutationAcceptViewResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &property.AcceptViewRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()
	return connections.PropertyService.AcceptView(context.Background(), request)
}

func PropertyMutationRejectViewResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &property.RejectViewRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()
	return connections.PropertyService.RejectView(context.Background(), request)
}

func PropertyMutationCompleteViewResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &property.CompleteViewRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()
	return connections.PropertyService.CompleteView(context.Background(), request)
}

func PropertyMutationFailViewResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &property.FailViewRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()
	return connections.PropertyService.FailView(context.Background(), request)
}

func PropertyMutationChangeBankDetails(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &property.ChangeBankDetailsRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	if remote, ok := params.Context.Value("remote").(string); ok {
		request.Remote = remote
	}
	request.User = user.GetAccount().GetIdValue()
	return connections.PropertyService.ChangeBankDetails(context.Background(), request)
}

func PropertyQueryRenderHeiDocuments(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &property.RenderHeiDocumentsRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()
	return connections.PropertyService.RenderHeiDocuments(context.Background(), request)
}

func PropertyQueryRenderRgiDocuments(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	request := &property.RenderRgiDocumentsRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()
	return connections.PropertyService.RenderRgiDocuments(context.Background(), request)
}

func adminMutationWithdrawPropertyResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}

	req := &property.WithdrawnRequest{}
	if err := util.UnmarshalArguments(params.Args, req); err != nil {
		return nil, err
	}

	request := &property.WithdrawnRequest{
		Id:        req.Id,
		User:      user.GetAccount().GetIdValue(),
		AdminSite: false,
	}

	return connections.PropertyService.ServiceWithdrawn(params.Context, request)

}
