package resolvers

import (
	"github.com/graphql-go/graphql"
	"gitlab.okta-solutions.com/mashroom/backend/auth/api"
	"gitlab.okta-solutions.com/mashroom/backend/common/graphql/util"
	"gitlab.okta-solutions.com/mashroom/backend/extras/api"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/connections"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/resolvers/errs"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/schema"
)

func init() {
	query := graphql.NewObject(graphql.ObjectConfig{
		Name:        "ServicesQuery",
		Description: "Services query root",
		Fields: graphql.Fields{
			"get": &graphql.Field{
				Description: "Get service by id",
				Args:        util.BindArguments(extras.GetRequest{}),
				Resolve:     ServicesQueryGetResolver,
				Type:        util.BindOutput(extras.GetResponse{}, "Service description", nil),
			},
			"list": &graphql.Field{
				Description: "Get services list",
				Args:        util.BindArguments(extras.ListRequest{}),
				Resolve:     ServicesQueryListResolver,
				Type:        util.BindOutput(extras.ListResponse{}, "Services list", nil),
			},
			"listwithprice": &graphql.Field{
				Description: "Get services list",
				Args:        util.BindArguments(extras.ListRequest{}),
				Resolve:     ServicesQueryListWithPriceResolver,
				Type:        util.BindOutput(extras.ListResponse{}, "Services list with calculated prices", nil),
			},
		},
	})

	schema.RootQueries["services"] = &graphql.Field{
		Type: query,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return query, nil
		},
	}

	mutation := graphql.NewObject(graphql.ObjectConfig{
		Name:        "ServicesMutation",
		Description: "Services mutation root",
		Fields: graphql.Fields{
			"add": &graphql.Field{
				Description: "Add service",
				Args:        util.BindArguments(extras.AddRequest{}),
				Resolve:     ServicesMutationAddResolver,
				Type:        util.BindOutput(extras.AddResponse{}, "Services add", nil),
			},
			"update": &graphql.Field{
				Description: "Update service",
				Args:        util.BindArguments(extras.UpdateRequest{}),
				Resolve:     ServicesMutationUpdateResolver,
				Type:        util.BindOutput(extras.UpdateResponse{}, "Service update", nil),
			},
			"delete": &graphql.Field{
				Description: "Delete service",
				Args:        util.BindArguments(extras.DeleteRequest{}),
				Resolve:     ServicesMutationDeleteResolver,
				Type:        util.BindOutput(extras.DeleteResponse{}, "Service delete", nil),
			},
		},
	})

	schema.RootMutations["services"] = &graphql.Field{
		Type: mutation,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return mutation, nil
		},
	}
}

func ServicesQueryGetResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &extras.GetRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.ExtrasService.Get(params.Context, request)
}

func ServicesQueryListResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &extras.ListRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.ExtrasService.List(params.Context, request)
}

func ServicesQueryListWithPriceResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &extras.ListRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.ExtrasService.ListWithPrice(params.Context, request)
}

func ServicesMutationAddResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	if !HasRole(user, ContentManagerRole) {
		return nil, errs.AuthError
	}

	request := &extras.AddRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.ExtrasService.Add(params.Context, request)
}

func ServicesMutationUpdateResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	if !HasRole(user, ContentManagerRole) {
		return nil, errs.AuthError
	}

	request := &extras.UpdateRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.ExtrasService.Update(params.Context, request)
}

func ServicesMutationDeleteResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value("user").(*auth.AuthResponse)
	if user == nil {
		return nil, errs.NoUserError
	}
	if !HasRole(user, ContentManagerRole) {
		return nil, errs.AuthError
	}

	request := &extras.DeleteRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.ExtrasService.Delete(params.Context, request)
}
