//go:generate go-bindata -pkg impl -ignore .*\.go .
//go:generate go generate ./resolvers
package impl

import (
	"bytes"
	"crypto/sha256"
	"encoding/json"
	"fmt"
	"github.com/eientei/wsgraphql"
	"github.com/eientei/wsgraphql/mutcontext"
	"github.com/gorilla/websocket"
	"gitlab.okta-solutions.com/mashroom/backend/auth/api"
	"gitlab.okta-solutions.com/mashroom/backend/common/generic/env"
	"gitlab.okta-solutions.com/mashroom/backend/common/generic/log"
	"gitlab.okta-solutions.com/mashroom/backend/document/api"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/connections"
	_ "gitlab.okta-solutions.com/mashroom/backend/gateway/impl/resolvers"
	_ "gitlab.okta-solutions.com/mashroom/backend/gateway/impl/resolvers/admin"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/rest"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/schema"
	"html/template"
	"io"
	"net/http"
	"os"
	"path/filepath"
	"strconv"
	"strings"
	"time"
)

const (
	MaxMultipartMemory   = 128 << 20
	MediaCacheDirEnv     = "MEDIA_CACHE_DIR"
	MediaCacheDirDefault = "/media"
)

var MediaCacheDir = MediaCacheDirDefault
var AdminSiteUrl = env.ResolveString("ADMIN_SITE_URL", "admin.staging.env.mashroom.com")

var page *template.Template

func init() {
	if v, ok := os.LookupEnv(MediaCacheDirEnv); ok {
		MediaCacheDir = v
	}
	err := os.MkdirAll(MediaCacheDir, os.FileMode(0755))
	if err != nil {
		panic(err)
	}

	if b, err := Asset("playground.html"); err != nil {
		panic(err)
	} else {
		page = template.Must(template.New("graphql").Parse(string(b)))
	}
}

func Playground(title string, endpoint string) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		err := page.Execute(w, map[string]string{
			"title":    title,
			"endpoint": endpoint,
			"version":  "1.6.2",
		})
		if err != nil {
			http.Error(w, err.Error(), 401)
			return
		}
	}
}

func middleware(writer http.ResponseWriter, request *http.Request) {
	AccessControlAllow(writer, request)
}

func middlewareHandler(h http.Handler) http.HandlerFunc {
	return func(writer http.ResponseWriter, request *http.Request) {
		middleware(writer, request)
		if request.Method != http.MethodOptions {
			h.ServeHTTP(writer, request)
		}
	}
}

func middlewareFunc(f http.HandlerFunc) http.HandlerFunc {
	return func(writer http.ResponseWriter, request *http.Request) {
		middleware(writer, request)
		if request.Method != http.MethodOptions {
			f(writer, request)
		}
	}
}

func Serve(addr string) {
	s, err := schema.CompileSchema()
	if err != nil {
		panic(err)
	}

	graphqlHandler, err := wsgraphql.NewServer(wsgraphql.Config{
		Schema: &s,
		Upgrader: &websocket.Upgrader{
			CheckOrigin:  func(r *http.Request) bool { return true },
			Subprotocols: []string{"graphql-ws"},
		},
		OnPlainInit: func(globalctx mutcontext.MutableContext, r *http.Request, w http.ResponseWriter) {
			globalctx.Set("globalctx", globalctx)
			var remote string
			if h := r.Header.Get("X-Forwarded-For"); len(h) > 0 {
				remote = h
			} else {
				remote = r.RemoteAddr
			}
			if idx := strings.Index(remote, ","); idx > 0 {
				remote = remote[:idx]
			}
			if idx := strings.LastIndex(remote, ":"); idx > 0 {
				remote = remote[:idx]
			}
			log.Debugf("r.Host: [%s] AdminSiteUrl: [%s]", r.Host, AdminSiteUrl)
			globalctx.Set("adminSite", strings.EqualFold(r.Host, AdminSiteUrl))
			globalctx.Set("remote", remote)
			if v := r.Header.Get("Authorization"); len(v) > 0 {
				globalctx.Set("access_token", v)
			}
			if v := r.Header.Get("authToken"); len(v) > 0 {
				globalctx.Set("access_token", v)
			}
		},
		OnConnect: func(globalctx mutcontext.MutableContext, parameters interface{}) error {
			globalctx.Set("user", (*auth.AuthResponse)(nil))
			if parameters != nil {
				mapparams := parameters.(map[string]interface{})
				if v, ok := mapparams["authToken"].(string); ok {
					globalctx.Set("access_token", v)
				}
			}

			if rawtoken := globalctx.Value("access_token"); rawtoken != nil {
				if token, ok := rawtoken.(string); ok {
					adminSite, _ := globalctx.Value("adminSite").(bool)
					resp, err := connections.AuthService.Auth(globalctx, &auth.AuthRequest{
						AccessToken: token,
						AdminSite:   adminSite,
					})
					if err != nil {
						resp = nil
					}

					globalctx.Set("user", resp)
				}
			}
			return nil
		},
	})
	if err != nil {
		panic(err)
	}

	http.HandleFunc("/", middlewareHandler(Playground("GraphQL playground", "/query")))
	http.HandleFunc("/query", middlewareHandler(graphqlHandler))
	http.HandleFunc("/media", middlewareFunc(MediaUploader))
	http.HandleFunc("/media/", middlewareFunc(MediaId))
	http.HandleFunc("/callback/hellosign", middlewareHandler(rest.AppHandler(rest.HellosignHandler)))
	http.HandleFunc("/callback/stripe", middlewareHandler(rest.AppHandler(rest.StripeHandler)))
	http.HandleFunc("/callback/tokenio", middlewareFunc(rest.TokenioHandler))
	http.HandleFunc("/api/property/lookup/", middlewareFunc(rest.IvrHandler))
	http.HandleFunc("/callback/twilio", middlewareFunc(rest.TwilioIvrHandler))
	http.HandleFunc("/callback/twilio/file/", middlewareFunc(rest.TwilioFileHandler))
	log.Fatal(http.ListenAndServe(addr, nil))
}

func Id2Filepath(id string) string {
	var path []rune
	for i := 0; i < 64; i += 4 {
		path = append(path,
			'/',
			rune(id[i]),
			rune(id[i+1]),
			rune(id[i+2]),
			rune(id[i+3]),
		)
	}
	return string(path)
}

func MediaId(writer http.ResponseWriter, request *http.Request) {
	id := strings.Replace(request.URL.Path, "/media/", "", 1)
	if len(id) == 0 {
		MediaUploader(writer, request)
		return
	}
	switch request.Method {
	case http.MethodGet:
		MediaDownloader(writer, request)
	case http.MethodPatch:
		MediaUpdater(writer, request)
	case http.MethodOptions:
		writer.WriteHeader(200)
	}
}

func MediaDownloader(writer http.ResponseWriter, request *http.Request) {
	id := strings.Replace(request.URL.Path, "/media/", "", 1)
	var path string
	inforeq := false
	if strings.Index(id, "info-") == 0 {
		id = strings.Replace(id, "info-", "", 1)
		inforeq = true
	}
	id = strings.TrimSuffix(id, filepath.Ext(id))
	if len(id) != 64 {
		http.Error(writer, "invalid id format", http.StatusBadRequest)
		return
	}

	info, err := connections.DocumentService.FileInfo(request.Context(), &document.FileInfoRequest{
		Id: id,
	})
	if err != nil {
		http.Error(writer, err.Error(), http.StatusInternalServerError)
		return
	}
	if inforeq {
		writer.Header().Set("Content-Type", "application/json")
		bs, err := json.Marshal(info.File)
		if err != nil {
			http.Error(writer, err.Error(), http.StatusInternalServerError)
			return
		}
		_, err = writer.Write(bs)
		if err != nil {
			http.Error(writer, err.Error(), http.StatusInternalServerError)
			return
		}
		return
	}

	path = MediaCacheDir + Id2Filepath(id)

	_, err = os.Stat(path)

	if os.IsNotExist(err) {
		success := false
		client, err := connections.DocumentService.FileDownload(request.Context(), &document.FileDownloadRequest{
			Id: id,
		})
		if err != nil {
			http.Error(writer, err.Error(), http.StatusInternalServerError)
			return
		}
		ev, err := client.Recv()
		if err != nil {
			http.Error(writer, err.Error(), http.StatusInternalServerError)
			return
		}
		if ev.Init == nil {
			http.Error(writer, "missing init", http.StatusInternalServerError)
			return
		}
		init := ev.Init.File
		idx := strings.LastIndex(path, "/")
		err = os.MkdirAll(path[:idx], os.FileMode(0755))
		if err != nil {
			http.Error(writer, err.Error(), http.StatusInternalServerError)
			return
		}

		f, err := os.OpenFile(path, os.O_RDWR|os.O_CREATE, os.FileMode(0644))
		if err != nil {
			http.Error(writer, err.Error(), http.StatusInternalServerError)
			return
		}
		defer func() {
			_ = f.Close()
			if !success {
				_ = os.Remove(path)
			}
		}()

		h := sha256.New()
		for {
			ev, err := client.Recv()
			if err == io.EOF {
				break
			}
			if err != nil {
				http.Error(writer, err.Error(), http.StatusInternalServerError)
				return
			}
			if ev.Data == nil {
				http.Error(writer, "missing data", http.StatusInternalServerError)
				return
			}

			_, err = f.Write(ev.Data.Data)
			if err != nil {
				http.Error(writer, err.Error(), http.StatusInternalServerError)
				return
			}

			_, err = h.Write(ev.Data.Data)
			if err != nil {
				http.Error(writer, err.Error(), http.StatusInternalServerError)
				return
			}
		}

		digest := h.Sum(nil)
		if !bytes.Equal(digest, init.Sha256) {
			http.Error(writer, "digest mismatch", http.StatusInternalServerError)
			return
		}
		success = true
	}

	writer.Header().Set("Content-Type", info.File.Mime)
	http.ServeFile(writer, request, path)
}

func MediaWatermarkUploader(writer http.ResponseWriter, request *http.Request) {
	fmt.Println("enter")
	if request.Method == http.MethodOptions {
		fmt.Println("opts")
		writer.WriteHeader(200)
		return
	}

	err := request.ParseMultipartForm(MaxMultipartMemory)
	if err != nil {
		fmt.Println("multipart err", err)
		http.Error(writer, err.Error(), http.StatusBadRequest)
		return
	}

	tempdir := os.TempDir() + "/tmp-" + strconv.FormatInt(time.Now().Unix(), 10)
	err = os.MkdirAll(tempdir, os.FileMode(0755))
	if err != nil {
		fmt.Println("err1", err)
		http.Error(writer, err.Error(), http.StatusInternalServerError)
		return
	}
	defer func() {
		_ = os.RemoveAll(tempdir)
	}()

	file, header, err := request.FormFile("file")
	if err != nil {
		fmt.Println("err2", err)
		http.Error(writer, err.Error(), http.StatusInternalServerError)
		return
	}
	defer func() {
		_ = file.Close()
	}()

	mime := header.Header.Get("Content-Type")
	tmp, err := os.OpenFile(tempdir+"/data", os.O_RDWR|os.O_CREATE, os.FileMode(0644))
	if err != nil {
		fmt.Println("err3", err)
		http.Error(writer, err.Error(), http.StatusInternalServerError)
		return
	}
	defer func() {
		_ = tmp.Close()
	}()

	size := int64(0)
	h := sha256.New()
	buf := make([]byte, document.ChunkSize)
	for {
		n, rerr := file.Read(buf)
		_, werr := tmp.Write(buf[:n])
		if werr != nil {
			fmt.Println("err4", err)
			http.Error(writer, werr.Error(), http.StatusInternalServerError)
			return
		}

		if rerr == io.EOF {
			break
		} else if rerr != nil {
			fmt.Println("err5", err)
			http.Error(writer, rerr.Error(), http.StatusInternalServerError)
			return
		}

		_, err := h.Write(buf[:n])
		if err != nil {
			fmt.Println("err6", err)
			http.Error(writer, err.Error(), http.StatusInternalServerError)
			return
		}

		size += int64(n)
	}

	if size != header.Size {
		fmt.Println("err7", err)
		http.Error(writer, "Invalid file size after transfer", http.StatusInternalServerError)
		return
	}

	digest := h.Sum(nil)
	client, err := connections.DocumentService.FileUpload(request.Context())
	if err != nil {
		fmt.Println("err8", err)
		http.Error(writer, err.Error(), http.StatusInternalServerError)
		return
	}

	finit := &document.File{
		Name:   header.Filename,
		Mime:   mime,
		Size:   size,
		Sha256: digest,
	}

	descs := request.MultipartForm.Value["description"]
	if len(descs) > 0 {
		finit.Description = descs[0]
	}

	err = client.Send(&document.FileEvent{
		Init: &document.FileInit{
			File: finit,
		},
	})
	if err != nil {
		fmt.Println("err9", err)
		http.Error(writer, err.Error(), http.StatusInternalServerError)
		return
	}

	_, err = tmp.Seek(0, io.SeekStart)
	if err != nil {
		fmt.Println("err10", err)
		http.Error(writer, err.Error(), http.StatusInternalServerError)
		return
	}

	read := int64(0)
	for {
		slice := buf
		if size-read < int64(len(buf)) {
			slice = buf[:size-read]
		}
		if len(slice) == 0 {
			break
		}

		n, err := io.ReadFull(tmp, slice)
		if err != nil && err != io.EOF && err != io.ErrUnexpectedEOF {
			fmt.Println("err11", err)
			http.Error(writer, err.Error(), http.StatusInternalServerError)
			return
		}

		read += int64(n)

		err = client.Send(&document.FileEvent{
			Data: &document.FileData{
				Data: slice,
			},
		})

		eof := err != nil && strings.Contains(err.Error(), "EOF")

		if eof {
			break
		}

		if err != nil {
			fmt.Println("err12", err)
			http.Error(writer, err.Error(), http.StatusInternalServerError)
			return
		}
	}

	resp, err := client.CloseAndRecv()
	if err != nil {
		fmt.Println("err13", err)
		http.Error(writer, err.Error(), http.StatusInternalServerError)
		return
	}
	bs, err := json.Marshal(resp.File)
	if err != nil {
		fmt.Println("err14", err)
		http.Error(writer, err.Error(), http.StatusInternalServerError)
		return
	}
	writer.Header().Set("Content-Type", "application/json")
	_, err = writer.Write(bs)
	if err != nil {
		fmt.Println("err15", err)
		http.Error(writer, err.Error(), http.StatusInternalServerError)
		return
	}
}

func MediaUploader(writer http.ResponseWriter, request *http.Request) {
	fmt.Println("enter")
	if request.Method == http.MethodOptions {
		fmt.Println("opts")
		writer.WriteHeader(200)
		return
	}

	err := request.ParseMultipartForm(MaxMultipartMemory)
	if err != nil {
		fmt.Println("multipart err", err)
		http.Error(writer, err.Error(), http.StatusBadRequest)
		return
	}

	tempdir := os.TempDir() + "/tmp-" + strconv.FormatInt(time.Now().Unix(), 10)
	err = os.MkdirAll(tempdir, os.FileMode(0755))
	if err != nil {
		fmt.Println("err1", err)
		http.Error(writer, err.Error(), http.StatusInternalServerError)
		return
	}
	defer func() {
		_ = os.RemoveAll(tempdir)
	}()

	file, header, err := request.FormFile("file")
	if err != nil {
		fmt.Println("err2", err)
		http.Error(writer, err.Error(), http.StatusInternalServerError)
		return
	}
	defer func() {
		_ = file.Close()
	}()

	mime := header.Header.Get("Content-Type")
	tmp, err := os.OpenFile(tempdir+"/data", os.O_RDWR|os.O_CREATE, os.FileMode(0644))
	if err != nil {
		fmt.Println("err3", err)
		http.Error(writer, err.Error(), http.StatusInternalServerError)
		return
	}
	defer func() {
		_ = tmp.Close()
	}()

	size := int64(0)
	h := sha256.New()
	buf := make([]byte, document.ChunkSize)
	for {
		n, rerr := file.Read(buf)
		_, werr := tmp.Write(buf[:n])
		if werr != nil {
			fmt.Println("err4", err)
			http.Error(writer, werr.Error(), http.StatusInternalServerError)
			return
		}

		if rerr == io.EOF {
			break
		} else if rerr != nil {
			fmt.Println("err5", err)
			http.Error(writer, rerr.Error(), http.StatusInternalServerError)
			return
		}

		_, err := h.Write(buf[:n])
		if err != nil {
			fmt.Println("err6", err)
			http.Error(writer, err.Error(), http.StatusInternalServerError)
			return
		}

		size += int64(n)
	}

	if size != header.Size {
		fmt.Println("err7", err)
		http.Error(writer, "Invalid file size after transfer", http.StatusInternalServerError)
		return
	}

	digest := h.Sum(nil)
	client, err := connections.DocumentService.FileUpload(request.Context())
	if err != nil {
		fmt.Println("err8", err)
		http.Error(writer, err.Error(), http.StatusInternalServerError)
		return
	}

	finit := &document.File{
		Name:   header.Filename,
		Mime:   mime,
		Size:   size,
		Sha256: digest,
	}

	descs := request.MultipartForm.Value["description"]
	if len(descs) > 0 {
		finit.Description = descs[0]
	}

	err = client.Send(&document.FileEvent{
		Init: &document.FileInit{
			File: finit,
		},
	})
	if err != nil {
		fmt.Println("err9", err)
		http.Error(writer, err.Error(), http.StatusInternalServerError)
		return
	}

	_, err = tmp.Seek(0, io.SeekStart)
	if err != nil {
		fmt.Println("err10", err)
		http.Error(writer, err.Error(), http.StatusInternalServerError)
		return
	}

	read := int64(0)
	for {
		slice := buf
		if size-read < int64(len(buf)) {
			slice = buf[:size-read]
		}
		if len(slice) == 0 {
			break
		}

		n, err := io.ReadFull(tmp, slice)
		if err != nil && err != io.EOF && err != io.ErrUnexpectedEOF {
			fmt.Println("err11", err)
			http.Error(writer, err.Error(), http.StatusInternalServerError)
			return
		}

		read += int64(n)

		err = client.Send(&document.FileEvent{
			Data: &document.FileData{
				Data: slice,
			},
		})

		eof := err != nil && strings.Contains(err.Error(), "EOF")

		if eof {
			break
		}

		if err != nil {
			fmt.Println("err12", err)
			http.Error(writer, err.Error(), http.StatusInternalServerError)
			return
		}
	}

	resp, err := client.CloseAndRecv()
	if err != nil {
		fmt.Println("err13", err)
		http.Error(writer, err.Error(), http.StatusInternalServerError)
		return
	}
	bs, err := json.Marshal(resp.File)
	if err != nil {
		fmt.Println("err14", err)
		http.Error(writer, err.Error(), http.StatusInternalServerError)
		return
	}
	writer.Header().Set("Content-Type", "application/json")
	_, err = writer.Write(bs)
	if err != nil {
		fmt.Println("err15", err)
		http.Error(writer, err.Error(), http.StatusInternalServerError)
		return
	}
}

func MediaUpdater(writer http.ResponseWriter, request *http.Request) {
	id := strings.Replace(request.URL.Path, "/media/", "", 1)
	id = strings.TrimSuffix(id, filepath.Ext(id))
	if len(id) != 64 {
		http.Error(writer, "invalid id format", http.StatusBadRequest)
		return
	}

	err := request.ParseMultipartForm(MaxMultipartMemory)
	if err != nil {
		http.Error(writer, err.Error(), http.StatusBadRequest)
		return
	}

	file := &document.File{
		Id: id,
	}

	descs := request.MultipartForm.Value["description"]
	if len(descs) > 0 {
		file.Description = descs[0]
	}

	resp, err := connections.DocumentService.FileUpdate(request.Context(), &document.FileUpdateRequest{
		File: file,
	})

	if err != nil {
		http.Error(writer, err.Error(), http.StatusInternalServerError)
		return
	}

	writer.Header().Set("Content-Type", "application/json")
	bs, err := json.Marshal(resp.File)
	if err != nil {
		http.Error(writer, err.Error(), http.StatusInternalServerError)
		return
	}
	_, err = writer.Write(bs)
	if err != nil {
		http.Error(writer, err.Error(), http.StatusInternalServerError)
		return
	}
	return
}

func AccessControlAllow(w http.ResponseWriter, r *http.Request) {
	w.Header().Set("Vary", "Authorization, authToken, Origin")

	headers := []string{
		"Authorization",
		"authToken",
		"Origin",
		"Authorization",
		"authToken",
		"Access-Control-Request-Headers",
		"Access-Control-Request-Method",
		"X-Forwarded-For",
		"X-Forwarded-Port",
		"X-Forwarded-Proto",
		"X-Requested-With",
		"Upgrade",
		"Connection",
		"Via",
		"Media-Range",
		"Optional",
		"If",
		"If-Match",
		"If-Modified-Since",
		"If-None-Match",
		"If-Range",
		"If-Unmodified-Since",
		"Keep-Alive",
		"Last-Modified",
		"MIME-Version",
		"ETag",
		"Accept",
		"Accept-CH",
		"Accept-Charset",
		"Accept-Datetime",
		"Accept-Encoding",
		"Accept-Ext",
		"Accept-Features",
		"Accept-Language",
		"Accept-Params",
		"Accept-Ranges",
		"User-Agent",
		"*",
	}
	if req := r.Header.Get("Access-Control-Request-Headers"); req != "" {
		found := false
		for _, m := range headers {
			if strings.EqualFold(m, req) {
				found = true
				break
			}
		}
		if !found {
			headers = append(headers, req)
		}
	}
	w.Header().Set("Access-Control-Allow-Headers", strings.Join(headers, ", "))

	methods := []string{
		"GET",
		"HEAD",
		"POST",
		"PUT",
		"DELETE",
		"CONNECT",
		"OPTIONS",
		"TRACE",
		"PATCH",
		"*",
	}
	if req := r.Header.Get("Access-Control-Request-Method"); req != "" {
		found := false
		for _, m := range methods {
			if strings.EqualFold(m, req) {
				found = true
				break
			}
		}
		if !found {
			methods = append(methods, req)
		}
	}
	w.Header().Set("Access-Control-Allow-Methods", strings.Join(methods, ", "))

	if origin := r.Header.Get("Origin"); origin != "" {
		w.Header().Set("Access-Control-Allow-Origin", origin)
	}
}
